import { csv } from 'd3';
import {
  select,
  json,
  pie,
  arc,
  scaleOrdinal,
  format
} from 'd3';

//async() function ensures that the function returns a value
const run = async () => {
  let data = await fetchData()
  let total = 0
  data.forEach(d=>{
      d.TotalDeaths = +d.TotalDeaths;
    total += d.TotalDeaths
  });
	render(data,total);
};

//Defining the colors that each value is going to represent
const sectionColors = [
  		"#4b4b4b", "#9acbc2", "#f2a0ea", "#014140", "#d9d5d5","#FFD700", "#B22222",
      "#0000FF", "#f58026", "#00b5cc" ];

//Github link for the dataset
const csvUrl = 'https://gist.githubusercontent.com/SonuSriN/0b26a35ad674e627182d1607f269b45f/raw/c836767d3590fd2448853451536d61ba3426f5bc/OceaniaCovidReport.csv';

//async() function ensures that the function returns a value
const fetchData = async () => {
  const data = await csv(csvUrl);
  return data;
};

//Select the SVG from the HTML body
const svg = select("svg");
const width = + svg.attr('width');
const height = + svg.attr('height');

//Format tick values of the axis
const xAxisTickFormat = number =>
  	format('.5~g')(number);

//Defining the domain and range of the pie chart
const render = (data, total) => {
  const pieData = pie().value(d=>d.TotalDeaths)(data);
  const colors = scaleOrdinal()
  	.domain(data)
  	.range(sectionColors);
  
//using d3.arc() to create circular chart using arc generator for the pie chart
	const segments = arc()
  	.innerRadius(0)
  	.outerRadius(170)
  	.padAngle(0)
  	.padRadius(0);
  
//using d3.arc() to create circular chart using arc generator for the labels
  const arcGenerator = d3.arc()
  .innerRadius(0)
  .outerRadius(190);
  
  const sections = svg.append("g")
  	.attr("transform", `translate(250,250)`)
  	.selectAll("path").data(pieData);

//passing the values of total deaths into the colors
  sections.enter()
    .append("path")
    .attr("d", segments)
  	.attr("fill", d => colors(d.data.TotalDeaths));
  
//adding Labels to the Pie Chart
  sections
    .enter()
    .append('text')
    .text(d => xAxisTickFormat((d.data.TotalDeaths)))
    .attr("transform", function(d) { return "translate(" + arcGenerator.centroid(d) + ")";  })
    .style("text-anchor", "end")
  	.style('fill', 'black')
    .style("font-size", 10);
  
//render legend for the data
  const legends = svg.append("g")
  	.attr("transform", "translate(500,100)")
    .selectAll(".legends").data(pieData);
  
  const legend = legends.enter().append("g").classed(".legends",true)
  	.attr("transform", (d,i)=>{
      return `translate(0,${(i+1)*30})`;
    });
  
//Formatting the legend for Total Deaths
  legend.append("rect").attr("width",20).attr("height",20)
  	.attr("fill", d => colors(d.data.TotalDeaths));
  
  legend.append("text")
  	.attr("x", 250)
  	.attr("y", 15)
  	.attr("class","legend_value")
  	.text(d=>xAxisTickFormat((d.data.TotalDeaths)));
  
//Formatting the legend for Countries 
  legend.append("text")
  	.attr("x", 25)
  	.attr("y", 15)
  	.attr("class","legend_text")
  	.text(d => d.data.Country);
  
}
run();
